function [L, D] = expG(h, B, Rinv, L0, D0, args, t0)
% Given G(P) = -P B Rinv B^T P, compute the low-rank factorization LDL^T of 
%
%   exp(hG)P = (I + h P B Rinv B^T)^{-1}P
%
% where P = L0 D0 L0^T.
%
% In the time-dependent case, h B Rinv B^T should be replaced by
% int_{t}^{t+h} {B(s)RinvB(s)' ds}.

%
% Copyright 2018 Tony Stillfjord
%
% This file is part of DREsplit.
% 
% DREsplit is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% DREsplit is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with DREsplit.  If not, see <http://www.gnu.org/licenses/>.
%

    I = eye(size(D0));
    L = L0;
    if nargin >= 6 && strcmp(args.impODE.method, 'timevarying')
        % Time-dependent case. Only difference is that instead of 
        % h*B*Rinv*B' we have int_{t}^{t+h} {B(s)RinvB(s)' ds}
        % We approximate this as LB*DB*LB' by quadrature. Note that the 
        % factor h is included in the approximation.
        % Could probably get away with lower order...
        [xj, weights] = exact_quadrature_parameters(h, 29);
        if nargin < 7
            t0 = 0;
        end
        xj = xj + t0; % Shift from [0, h] t0 [t0, t0+h]

%         % Solving for X:
%         Bfun = @(t) args.M(t)\B(t);
%         LB = cell2mat(arrayfun(Bfun, xj', 'UniformOutput', false));

%         Solving for Xhat:
        LB = cell2mat(arrayfun(B, xj', 'UniformOutput', false));
        
        DB = kron(diag(weights), Rinv*speye(size(LB,2) / length(xj)));
        [LB, DB] = column_compress_LDLT(LB, DB, args.CC_tol);
        D = (I + D0*L0'*(LB*(DB*(LB'*L0))))\D0;    
    else
        D = (I + h*D0*L0'*(B*(Rinv*(B'*L0))))\D0;
    end

end